==> lupus_ce_renderer-2.0.0.tar.gz: lupus_ce_renderer/src/EventSubscriber/CustomElementsViewSubscriber.php <==
<?php

namespace Drupal\lupus_ce_renderer\EventSubscriber;

use drunomics\ServiceUtils\Core\Routing\CurrentRouteMatchTrait;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Site\Settings;
use Drupal\custom_elements\CustomElement;
use Drupal\lupus_ce_renderer\CustomElementsRenderer;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\GetResponseForControllerResultEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * View subscriber that renders custom elements into responses.
 */
class CustomElementsViewSubscriber implements EventSubscriberInterface {

  use CurrentRouteMatchTrait;

  /**
   * The custom element renderer.
   *
   * @var \Drupal\lupus_ce_renderer\CustomElementsRenderer
   */
  protected $customElementRenderer;

  /**
   * Constructs the object.
   *
   * @param \Drupal\lupus_ce_renderer\CustomElementsRenderer $customElementRenderer
   *   The custom element renderer.
   */
  public function __construct(CustomElementsRenderer $customElementRenderer) {
    $this->customElementRenderer = $customElementRenderer;
  }

  /**
   * Sets a response given a custom element.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseForControllerResultEvent $event
   *   The event to process.
   */
  public function onKernelView(GetResponseForControllerResultEvent $event) {
    $request = $event->getRequest();
    // We need to render error pages (404,403) via custom_elements as requested
    // or configured. If there is 404/403 error and custom_elements is the
    // default format, there won't be a custom-elements response,
    // because there is no custom-elements route available during routing,
    // therefore we need to ensure responses get rendered correctly here.
    if (Settings::get('lupus_ce_renderer_enable') === TRUE || $request->getRequestFormat() === 'custom_elements') {
      $this->handleClientErrorResponses($event);
    }
    $result = $event->getControllerResult();

    if ($result instanceof Response) {
      return;
    }

    // Render the controller result into a response if it's a render array.
    if ($result instanceof CustomElement) {
      $default_content_format = Settings::get('lupus_ce_renderer_default_format', CustomElementsRenderer::CONTENT_FORMAT_MARKUP);
      $content_format = $request->query->get('_content_format', $default_content_format);
      $wrapper_format = $request->query->get('_select');
      $response = $this->customElementRenderer->renderResponse($result, $content_format, $wrapper_format);

      // Add cache metadata for query arguments.
      if ($response instanceof CacheableResponseInterface) {
        $cache_metadata = (new CacheableMetadata())->setCacheContexts([
          'url.query_args:_content_format',
          'url.query_args:_select',
        ]);
        $response->addCacheableDependency($cache_metadata);
      }
      $event->setResponse($response);
    }
    elseif ($request->getRequestFormat() == 'custom_elements') {
      // Custom elements requested, but the route does not handle it.
      $event->setResponse(new Response('Route does not render into custom elements.', Response::HTTP_NOT_IMPLEMENTED));
    }
  }

  /**
   * Handles http 4xx responses gracefully.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseForControllerResultEvent $event
   *   The event to process.
   *
   * @see \Drupal\lupus_ce_renderer\EventSubscriber\CustomElementsHttpExceptionSubscriber
   */
  protected function handleClientErrorResponses(GetResponseForControllerResultEvent $event) {
    // Gracefully handle 401, 403 and 404 routes with a simple #markup key.
    $result = $event->getControllerResult();
    if (!$result instanceof CustomElement) {
      $status_code = $event->getRequest()->get('_exception_statuscode');
      if (strpos($this->getCurrentRouteMatch()->getRouteName(), 'system.4') === 0) {
        $event->setControllerResult(CustomElement::create('drupal-markup')
          ->setSlot('default', $result['#markup'])
        );
      }
      elseif (isset($status_code) && in_array($status_code, [401, 403, 404])) {
        $event->setControllerResult(CustomElement::create('drupal-markup')
          ->setSlotFromRenderArray('default', $result)
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Run before MainContentViewSubscriber so we can handle 40x responses.
    $events[KernelEvents::VIEW][] = ['onKernelView', 100];
    return $events;
  }
}

==> lupus_ce_renderer-2.1.0.tar.gz: lupus_ce_renderer/src/EventSubscriber/CustomElementsViewSubscriber.php <==
<?php

namespace Drupal\lupus_ce_renderer\EventSubscriber;

use Symfony\Component\HttpKernel\Event\ViewEvent;
use drunomics\ServiceUtils\Core\Routing\CurrentRouteMatchTrait;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Site\Settings;
use Drupal\custom_elements\CustomElement;
use Drupal\lupus_ce_renderer\CustomElementsRenderer;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * View subscriber that renders custom elements into responses.
 */
class CustomElementsViewSubscriber implements EventSubscriberInterface {

  use CurrentRouteMatchTrait;

  /**
   * The custom element renderer.
   *
   * @var \Drupal\lupus_ce_renderer\CustomElementsRenderer
   */
  protected $customElementRenderer;

  /**
   * Constructs the object.
   *
   * @param \Drupal\lupus_ce_renderer\CustomElementsRenderer $customElementRenderer
   *   The custom element renderer.
   */
  public function __construct(CustomElementsRenderer $customElementRenderer) {
    $this->customElementRenderer = $customElementRenderer;
  }

  /**
   * Sets a response given a custom element.
   *
   * @param \Symfony\Component\HttpKernel\Event\ViewEvent $event
   *   The event to process.
   */
  public function onKernelView(ViewEvent $event) {
    $request = $event->getRequest();

    // Errors are handled by CustomElementsHttpExceptionSubscriber already.
    // But if there is no custom-error page configured, the system-default
    // error pages apply, which render in HTML. We handle this case here.
    // @todo: Re-factor this to provide custom-enabled versions of system
    // module provided routes (system.4**) instead and remove special-handling
    // here.
    if ($request->getRequestFormat() === 'custom_elements' && !$event->getControllerResult() instanceof CustomElement) {
      $this->handleClientErrorResponses($event);
    }

    $result = $event->getControllerResult();
    if ($result instanceof Response) {
      return;
    }

    // Render the controller result into a response if it's a render array.
    if ($result instanceof CustomElement) {
      $default_content_format = Settings::get('lupus_ce_renderer_default_format', CustomElementsRenderer::CONTENT_FORMAT_MARKUP);
      $content_format = $request->query->get('_content_format', $default_content_format);
      $wrapper_format = $request->query->get('_select');
      $response = $this->customElementRenderer->renderResponse($result, $content_format, $wrapper_format);

      // Add cache metadata for query arguments.
      if ($response instanceof CacheableResponseInterface) {
        $cache_metadata = (new CacheableMetadata())->setCacheContexts([
          'url.query_args:_content_format',
          'url.query_args:_select',
        ]);
        $response->addCacheableDependency($cache_metadata);
      }
      $event->setResponse($response);
    }
    elseif ($request->getRequestFormat() == 'custom_elements') {
      // Custom elements requested, but the route does not handle it.
      $event->setResponse(new Response('Route does not render into custom elements.', Response::HTTP_NOT_IMPLEMENTED));
    }
  }

  /**
   * Handles http 4xx responses gracefully.
   *
   * @param \Symfony\Component\HttpKernel\Event\ViewEvent $event
   *   The event to process.
   *
   * @see \Drupal\lupus_ce_renderer\EventSubscriber\CustomElementsHttpExceptionSubscriber
   */
  protected function handleClientErrorResponses(ViewEvent $event) {
    // Gracefully handle 401, 403 and 404 routes with a simple #markup key.
    $result = $event->getControllerResult();
    if (!$result instanceof CustomElement) {
      $status_code = $event->getRequest()->get('_exception_statuscode');
      if (strpos($this->getCurrentRouteMatch()->getRouteName(), 'system.4') === 0) {
        $event->setControllerResult(CustomElement::create('drupal-markup')
          ->setSlot('default', $result['#markup'])
        );
      }
      elseif (isset($status_code) && in_array($status_code, [401, 403, 404])) {
        $event->setControllerResult(CustomElement::create('drupal-markup')
          ->setSlotFromRenderArray('default', $result)
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Run before MainContentViewSubscriber so we can handle 40x responses.
    $events[KernelEvents::VIEW][] = ['onKernelView', 100];
    return $events;
  }

}
